#include <stdio.h>
#include <string.h>
#include <net/xilnet_config.h>
#include <net/xilsock.h>
#include <net/udp.h>
#include <net/in.h>
#include <dhcp.h>


//---------contant definition -------------------

#define PKT_SIZE 			1024
#define INADDR_BROADCAST   (unsigned long)0xffffffff	

#define DHCP_DONE 		10
#define DHCP_DISCOVER 	1
#define DHCP_OFFER		2
#define DHCP_REQUEST	3
#define DHCP_ACK		5
#define DHCP_RENEW		11


//---------global constant declaration----------------
const char broadcast_haddr[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};	
const char mac_addr[6] = {0x00, 0xE0,0x98, 0x8F, 0xE9, 0x3F};	
const char magic_cookie[4] = {99, 130, 83, 99};			//DHCP magic cookie
const char xid[4] = {152, 203, 142, 89};				// transaction id, fixed
const char hostname[8] = "mbserver";


//---------global variable declaration-----------------
int dhcp_socket;						
unsigned char dhcp_state; 				// state machine variable
struct sockaddr_in addr;				//DHCP client address
struct sockaddr_in dhcp_to; 			//DHCP server address

unsigned char server_ip[4];
unsigned char myip[4] = {0,0,0,0};
unsigned char dns1[4] = {0,0,0,0};
unsigned char dns2[4] = {0,0,0,0};
unsigned char subnet_mask[4];
unsigned char router[4];
unsigned int  dhcp_lease_time;



//---------function prototyping-------------------------


void dhcp_tx(void);
void dhcp_rx(void);
void dhcp_dump(unsigned char *buffer,int size);
int  dhcp_sendto(int s, unsigned char* buf, unsigned int len, 
				 struct sockaddr* to, unsigned int tolen);


void dhcp_get_ip(unsigned char *ip_address)
{
	
	//----setup dhcp socket-------------
    dhcp_socket = xilsock_socket(AF_INET, SOCK_DGRAM, IP_PROTO_UDP); 
    if (dhcp_socket == -1) 
	{
		xil_printf("DHCP socket() error \n");
      	exit(1);
   	}

	//----setup dhcp addresses----------
	addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = INADDR_ANY;			
    addr.sin_port = 68;							//standard dhcp client port

	dhcp_to.sin_family = AF_INET;
	dhcp_to.sin_addr.s_addr = INADDR_BROADCAST;	// broadcast
	dhcp_to.sin_port = 67; 						//standard dhcp server port

	
	//----bind dhcp socket----------------
    if ( xilsock_bind(dhcp_socket, (struct sockaddr *)&addr, sizeof(struct sockaddr)) == -1 ) 
    {
       xil_printf("DHCP bind error \n");
       exit(1);
    }
	
	//----set initial state--------------
	dhcp_state = DHCP_DISCOVER;

	
	//-----run state machine ------------
	while (dhcp_state != DHCP_DONE)
	{
		switch (dhcp_state)
		{
			case DHCP_DISCOVER:
				dhcp_tx();
				break;
			case DHCP_OFFER:
				dhcp_rx();
				break;
			case DHCP_REQUEST:
				dhcp_tx();
				break;
			case DHCP_ACK:
				dhcp_rx();
				break;
		}
	}
		
	xilsock_close(dhcp_socket);		
	memcpy(ip_address, myip, 4);

}


#define ADDR_TYPE_OFFSET 	1
#define ADDR_LEN_OFFSET 	2
#define HOPS_OFFSET 		3
#define TRANS_ID_OFFSET 	4
#define ELAPSED_T_OFFSET 	8
#define FLAGS_OFFSET		10
#define CIADDR_OFFSET		12
#define YIADDR_OFFSET		16
#define SIADDR_OFFSET		20
#define GIADDR_OFFSET		24
#define CHADDR_OFFSET		28
#define SERVER_HOST_OFFSET	44
#define BOOT_FILE_OFFSET	108
#define MAGIC_COOKIE_OFFSET	236
#define DHCP_OPTION_OFFSET	240



void dhcp_tx(void)
{
	unsigned char dhcp_pkt[PKT_SIZE];
	int pkt_len;							//packet lenght				
	

	// DHCP Message
	memset(dhcp_pkt, 						1	,1);// DHCP request
	memset(dhcp_pkt + ADDR_TYPE_OFFSET, 	1	,1);// ethernet hardware
	memset(dhcp_pkt + ADDR_LEN_OFFSET, 		6	,1);// assume ethernet only, 6 byte address
	memset(dhcp_pkt + HOPS_OFFSET, 			0	,1);// zero hop count
	memcpy(dhcp_pkt + TRANS_ID_OFFSET, 		xid	,4);// transaction ID 
	memset(dhcp_pkt + ELAPSED_T_OFFSET, 	0	,2);// no elapsed time
	memset(dhcp_pkt + FLAGS_OFFSET, 		0	,2);// no flag
	memcpy(dhcp_pkt + CIADDR_OFFSET, 		0	,4);// client address yet
	memset(dhcp_pkt + YIADDR_OFFSET, 		0	,4);// not applicable
	memset(dhcp_pkt + SIADDR_OFFSET, 		0	,4);// not applicable
	memset(dhcp_pkt + GIADDR_OFFSET, 		0	,4);// not applicable

	memcpy(dhcp_pkt + CHADDR_OFFSET,		mac_addr		,6 );// MAC address
	memset(dhcp_pkt + CHADDR_OFFSET+6,		0				,10);// zero rest of the field
	memset(dhcp_pkt + SERVER_HOST_OFFSET,	0				,64);//aero everything
	memset(dhcp_pkt + BOOT_FILE_OFFSET,		0				,128);//aero everything
	memcpy(dhcp_pkt + MAGIC_COOKIE_OFFSET,  magic_cookie	,4 );// DHCP Magic cookie

	// DHCP Options
	memset(dhcp_pkt + DHCP_OPTION_OFFSET, 	53			,1);// Message type
	memset(dhcp_pkt + DHCP_OPTION_OFFSET+1, 1			,1);// 
	memset(dhcp_pkt + DHCP_OPTION_OFFSET+2, dhcp_state	,1);// 
	
	memset(dhcp_pkt + DHCP_OPTION_OFFSET+3,	61			,1);// Client ID
	memset(dhcp_pkt + DHCP_OPTION_OFFSET+4, 7			,1);// 
	memset(dhcp_pkt + DHCP_OPTION_OFFSET+5, 1			,1);// ethernet
	memcpy(dhcp_pkt + DHCP_OPTION_OFFSET+6,	mac_addr	,6);// MAC address

	memset(dhcp_pkt + DHCP_OPTION_OFFSET+12,12			,1);// Hostname
	memset(dhcp_pkt + DHCP_OPTION_OFFSET+13,8			,1);// 
	memcpy(dhcp_pkt + DHCP_OPTION_OFFSET+14,hostname	,8);// 

	if (dhcp_state == DHCP_REQUEST)		// Sending DHCP Request?							
	{
		memset(dhcp_pkt + DHCP_OPTION_OFFSET+22,54			,1);// DHCP server IP addr
		memset(dhcp_pkt + DHCP_OPTION_OFFSET+23,4			,1);// 
		memcpy(dhcp_pkt + DHCP_OPTION_OFFSET+24,server_ip	,4);// 

		memset(dhcp_pkt + DHCP_OPTION_OFFSET+28,50			,1);// requested IP addr
		memset(dhcp_pkt + DHCP_OPTION_OFFSET+29,4			,1);// 
		memcpy(dhcp_pkt + DHCP_OPTION_OFFSET+30,myip		,4);// 

		memset(dhcp_pkt + DHCP_OPTION_OFFSET+34,255			,1); // end of DHCP options

		pkt_len = DHCP_OPTION_OFFSET+35;
		dhcp_state = DHCP_ACK;
	}
	else
	{
		memset(dhcp_pkt + DHCP_OPTION_OFFSET+22,255			,1); // end of DHCP options

		pkt_len = DHCP_OPTION_OFFSET+22;
		dhcp_state = DHCP_OFFER;	
									
	}

	dhcp_dump(dhcp_pkt,pkt_len);		
	dhcp_sendto(dhcp_socket, dhcp_pkt, pkt_len, (struct sockaddr *) &dhcp_to, sizeof(dhcp_to)); 
		
}



void dhcp_rx(void)
{
	unsigned char dhcp_pkt[PKT_SIZE];			//holds received dhcp packet
    struct sockaddr_in dhcp_from;				//hdcp server address
    unsigned int fromlen = sizeof(dhcp_from);
    int bytes_recvd;
	unsigned char myip_string[4]; 				// for use with xilnet library		
	unsigned char buf[4];
	unsigned char type;		//dhcp type
	int j;					//dhcp packet buffer index



	//------get dhcp packet--------------------------------------------------
	xilnet_ip_init("255.255.255.255");	// work around to reciever IP broadcast messages
	bytes_recvd=xilsock_recvfrom(dhcp_socket,dhcp_pkt,PKT_SIZE,(struct sockaddr *)&dhcp_from,&fromlen);	
	xilnet_ip_init("0.0.0.0");			// set ip address back to 0.0.0.0
   

	//--------process packet information --------------------------
	if(bytes_recvd < 0)
		return;

	// check to ensure valid HDCP message
	if ( ( dhcp_pkt[0] 						== 	2)  	&&		// Boot reply
		 ( dhcp_pkt[ADDR_LEN_OFFSET]		==	6)		&&		// ethernet hardware
		 ( memcmp(magic_cookie, dhcp_pkt + MAGIC_COOKIE_OFFSET, 4) == 0) &&	//DHCP message
		 ( memcmp(mac_addr, dhcp_pkt + CHADDR_OFFSET, 4)==0)	// correct mac address
	   )
 		
	{
		j = 240;	//start of option portion
		
		// parse dhcp options
		while (j<bytes_recvd && dhcp_pkt[j]!=255) 
		{

			switch (dhcp_pkt[j]) 
			{
			case 1:		// subnet option
	    		memcpy(subnet_mask, dhcp_pkt+j+2, 4);
	    		break;
			case 3:		// router ip
	    		memcpy(router, dhcp_pkt+j+2, 4);
	    		break;	
			case 6: 	// domain name server
				memcpy(dns1, dhcp_pkt+j+2, 4);
				memcpy(dns2, dhcp_pkt+j+6, 4);
				break;	
			case 51:	// lease time
				memcpy(buf, dhcp_pkt+j+2, 4);
				dhcp_lease_time = atoi(buf);
	    		break;
			case 53:	// message type	    	
				type = dhcp_pkt[j+2];
	   			break;
			case 54:	// server ip
	    		memcpy(server_ip, dhcp_pkt+j+2, 4);
	    		break;
			}

			j+=dhcp_pkt[j+1]+2;
		}

		// trying to get a offer, and got an offer
		if ((dhcp_state == DHCP_OFFER) && (type == DHCP_OFFER) )
		{
			memcpy(myip, dhcp_pkt+ YIADDR_OFFSET, 4);		//get my ip address offer
			dhcp_state = DHCP_REQUEST;						//move on to request state						
		}
		else if ((dhcp_state == DHCP_ACK) && (type == DHCP_ACK))
		{
			sprintf(myip_string, "%d.%d.%d.%d", myip[0], myip[1], myip[2], myip[3]);
			xilnet_ip_init(myip_string);
			dhcp_state = DHCP_DONE;							//done
		}


		dhcp_dump(dhcp_pkt, bytes_recvd);
    }
}
		


void dhcp_dump(unsigned char *buffer,int size) 
{
    
	int j = 0;

	printf("packet %d bytes\n",size);

    while (j<size) 
	{
		printf("%02x ",buffer[j]);
		if (j%16==15) printf("\n");
		j++;
    }

    printf("\n");

    printf("Op: 		%d\n",buffer[0]);
    printf("htype: 		%d\n",buffer[1]);
    printf("hlen: 		%d\n",buffer[2]);
	printf("hops: 		%d\n",buffer[3]);

    printf("xid: 		%02x%02x%02x%02x\n", 
		buffer[4],buffer[5],buffer[6],buffer[7]);
    printf("secs: 		%d\n",255*buffer[8]+buffer[9]);
    printf("flags: 		%x\n",255*buffer[10]+buffer[11]);

    printf("ciaddr: 	%d.%d.%d.%d\n",
	    buffer[12],buffer[13],buffer[14],buffer[15]);
    printf("yiaddr: 	%d.%d.%d.%d\n",
	    buffer[16],buffer[17],buffer[18],buffer[19]);
    printf("siaddr:		%d.%d.%d.%d\n",
	    buffer[20],buffer[21],buffer[22],buffer[23]);
    printf("giaddr: 	%d.%d.%d.%d\n",
	    buffer[24],buffer[25],buffer[26],buffer[27]);
    printf("chaddr: %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n",
	    buffer[28],buffer[29],buffer[30],buffer[31],
	    buffer[32],buffer[33],buffer[34],buffer[35],
	    buffer[36],buffer[37],buffer[38],buffer[39],
	    buffer[40],buffer[41],buffer[42],buffer[43]);
    printf("sname : %s.\n",buffer+44);
    printf("fname : %s.\n",buffer+108);

    j=236;
    j+=4;	/* cookie */

    while (j<size && buffer[j]!=255) {
		//printf("option 		%d %s\n",buffer[j],dhcp_options[buffer[j]]);

		switch (buffer[j]) 
		{
		case 1:
	    	printf("\tSubnet mask: %d.%d.%d.%d\n",
			buffer[j+2],buffer[j+3],buffer[j+4],buffer[j+5]);
	    	break;
		case 3:
	    	printf("\tRouter: %d.%d.%d.%d\n",
			buffer[j+2],buffer[j+3],buffer[j+4],buffer[j+5]);
	    	break;
		case 12:
	    	printf("\tHost Name: %c%c%c%c%c%c%c%c\n",
			buffer[j+2],buffer[j+3],buffer[j+4],buffer[j+5],
			buffer[j+6],buffer[j+7],buffer[j+8],buffer[j+9]);
	    	break;
		case 50:
	    	printf("\tRequested IP address: %d.%d.%d.%d\n",
			buffer[j+2],buffer[j+3],buffer[j+4],buffer[j+5]);
	    	break;
		case 51:
	    	printf("\tIP Address Lease Time: %d.\n",
			(int) buffer[j+2]*256^3 + buffer[j+3]*256^2 + buffer[j+4]*256 + buffer[j+5]);
	    	break;
		case 53:
	    	//printf("\tDHCP message type: %d (%s)\n",
			//buffer[j+2],dhcp_message_types[buffer[j+2]]);
			printf("\tDHCP message type: %d \n", buffer[j+2]);
	   		break;
		/*case 54:
	    	memcpy(serveridentifier,buffer+j+2,4);
	    	printf("\tServer identifier: %d.%d.%d.%d\n",
			serveridentifier[0],serveridentifier[1],
			serveridentifier[2],serveridentifier[3]);
	    	break;
		*/
		case 61:
	    	printf("\tClient identifier: %02x%02x%02x%02x%02x%02x\n",
			buffer[j+2],buffer[j+3],buffer[j+4],
			buffer[j+5],buffer[j+6],buffer[j+7]);
	    	break;
		}


		j+=buffer[j+1]+2;
    }
}

	
int dhcp_sendto(int s, unsigned char* buf, unsigned int len, struct sockaddr* to, unsigned int tolen)
{

   struct xilnet_udp_conn *conn;
   unsigned int dstaddr = ((struct sockaddr_in*)to)->sin_addr.s_addr;
   
   conn = xilsock_sockets[s].conn.udp_conn;
   conn->dst_ip[0] = (unsigned char) ((dstaddr >> 24) & 0xFF);
   conn->dst_ip[1] = (unsigned char) ((dstaddr >> 16) & 0xFF);
   conn->dst_ip[2] = (unsigned char) ((dstaddr >> 8) & 0xFF);
   conn->dst_ip[3] = (unsigned char) ((dstaddr) & 0xFF);
   conn->dst_port = ((struct sockaddr_in*)to)->sin_port;
   
   if (!conn) {
      print("sendto: no such socket \n");
      return -1;
   }
   
   if (buf != sendbuf) {
      memset(sendbuf, 0, LINK_FRAME_LEN);
      memcpy(sendbuf+LINK_HDR_LEN+IP_HDR_LEN*4+UDP_HDR_LEN, buf, len); 
   }
   
   // calls to udp stack 
   xilnet_udp_header(conn, sendbuf+LINK_HDR_LEN+IP_HDR_LEN*4, len+UDP_HDR_LEN);

   xilnet_ip_header(sendbuf+LINK_HDR_LEN, len+UDP_HDR_LEN+IP_HDR_LEN*4, 
   					IP_PROTO_UDP, conn->dst_ip);

   xilnet_eth_send_frame(sendbuf, len+UDP_HDR_LEN+IP_HDR_LEN*4+ETH_HDR_LEN, 
   						conn->dst_ip, broadcast_haddr, ETH_PROTO_IP);

   return len;
}		

