
#include <ip_out.h>

const char hlen = IP_HDR_LEN*4 + LINK_HDR_LEN; // header length

void ip_output(int s, unsigned char* buf, unsigned int frame_len)
{
	
	unsigned int off, len, error = 0; 
	struct ip *mhip;
	struct frag_list *m, *m0, *m_tmp;

	struct xilnet_tcp_conn *conn;
	conn = xilsock_sockets[s].conn.tcp_conn;

	/*
	 * If small enough for ethernet, send directly.
	 */
	if (frame_len <= MTU) 
	{
		xilnet_eth_send_frame(buf, frame_len, conn->dst_ip, NULL, ETH_PROTO_IP);
		return;
	}
	
	len = (MTU - hlen) &~ 7;	// round down to 8-byte boundary

	/*
	 * Loop through length of segment after first fragment,
	 * make new header and copy data of each part and link onto chain.
	 */
	
	
	m = malloc(sizeof(struct frag_list));
	if (m)
	{
		m->next = (struct frag_list *)NULL;
		memset(m->data, 0, MTU);
	}
	else
	{
		printf("malloc error \n");
		error++;
		goto sendorfree;
	}

	m0 = m;

	for (off = hlen; off < frame_len; off += len) 
	{
		//------copy ip header---------------
		mhip = (m->data+LINK_HDR_LEN);			//point to begining of ip header
		memcpy(mhip, buf+LINK_HDR_LEN, IP_HDR_LEN*4);		//copy ip header
	
		//------modify ip header for fragmentation---------------	

		//ip_off field, ignore the case where the packet is an oversized fragment of a bigger packet 
		mhip->ip_off = ((off - hlen) >> 3);
		if (off + len >= frame_len)	
			len = frame_len - off;
		else
			mhip->ip_off |= IP_MF;
			
		
		//new ip_len
		mhip->ip_len = len + IP_HDR_LEN*4;	// big endian already, no need to swap byte
		
		//new checksum
		mhip->ip_sum = 0;
		mhip->ip_sum = xilnet_ip_calc_chksum(m->data+LINK_HDR_LEN, IP_HDR_LEN*4);;
			
		//------copy a fragment of IP datagram ------------------
		memcpy((unsigned char *)m->data + hlen, buf + off ,len);
		m->data_len = hlen + len;
		
        //------create a new node, and connect the linked list ------------	
		if(mhip->ip_off & IP_MF)								// more nodes to connect?
		{
			m_tmp = malloc(sizeof(struct frag_list));
			if (m_tmp){
				m->next = m_tmp;
				m = m_tmp;
			}
			else{
				printf("malloc error \n");
				error++;
				goto sendorfree;
			}
		}
		else													// end of the list
			m->next = (struct frag_list *) NULL;
	}

sendorfree:
	for (m = m0; m; m = m0) {
		m0 = m->next;
		if (error == 0)
			xilnet_eth_send_frame(m->data, m->data_len, conn->dst_ip, NULL, ETH_PROTO_IP);
		free(m);		// better if the list can be reused, buf for simplicity this is skipped.
	}

}
