
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity lf_decode is  --USER-- change entity name
  generic
  (
    C_MEM_FILL_ADDR_START : std_logic_vector(0 to 31) := conv_std_logic_vector(16#FFFFFFFF#,32);
    C_MEM_FILL_ADDR_END   : std_logic_vector(0 to 31) := conv_std_logic_vector(16#FFFFFFFF#,32);
    C_REPEAT              : INTEGER := 1;
    C_OPB_AWIDTH          : INTEGER := 32;
    C_OPB_DWIDTH          : INTEGER := 32;
    C_DEV_BURST_ENABLE    : INTEGER := 0;
    C_DEV_MAX_BURST_SIZE  : INTEGER := 64;
    C_FBADDR              : std_logic_vector(0 to 31) := conv_std_logic_vector(16#00020000#,32);
    C_FAMILY              : string  := "virtex2"
  );

  port
  (
    Bus2IP_Clk        : in  std_logic;
    Bus2IP_Data       : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    Bus2IP_Freeze     : in  std_logic;
    Bus2IP_IPMstTrans : in  std_logic;
    Bus2IP_MstAck     : in  std_logic;
    Bus2IP_MstRetry   : in  std_logic;
    Bus2IP_MstError   : in  std_logic;
    Bus2IP_MstTimeOut : in  std_logic;
    Bus2IP_MstLastAck : in  std_logic;
    Bus2IP_Reset      : in  std_logic;
    IP2Bus_Addr       : out std_logic_vector(0 to C_OPB_AWIDTH - 1 ) := (others => '0');
    IP2Bus_Data       : out std_logic_vector(0 to C_OPB_DWIDTH -1) := (others => '0');
    IP2Bus_MstBE      : out std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 ) := (others => '0');
    IP2Bus_MstReq     : out std_logic := '0';
    IP2Bus_MstRNW     : out std_logic := '0';
    IP2Bus_MstBurst   : out std_logic := '0';
    IP2Bus_MstBusLock : out std_logic := '0';
    --USER-- add user I/Os to port list

    YCrCb_in          : in std_logic_vector(9 downto 0);
    vid_clk           : in std_logic;
    write_ena         : in std_logic;

    A : out std_logic;
    B : out std_logic;
    C : out std_logic
  );
end entity lf_decode;

architecture lf_decode_rtl of lf_decode is

  -- added this 32-bit register signal
  signal pixeldata            : std_logic_vector (31 downto 0) := X"00000000";
  -- added reset and counter signal for frame capture
  signal vidcount	            : std_logic_vector (0 to 31);

-- NEED TO FIGURE OUT WHERE THE CLOCK INCREMENTS... 


  signal H_rg                 : std_logic_vector (4 downto 0);
  signal H_rising             : std_logic;
  signal TRS                  : std_logic;
  signal V_falling            : std_logic;
  signal V_rising             : std_logic;
  signal YCrCb_rg1            : std_logic_vector (9 downto 0);
  signal YCrCb_rg2            : std_logic_vector (9 downto 0);
  signal YCrCb_rg3            : std_logic_vector (9 downto 0);
  signal YCrCb_rg4            : std_logic_vector (9 downto 0);
  signal YCrCb_rg5            : std_logic_vector (9 downto 0);

  signal YCrCb_rg6            : std_logic_vector (9 downto 0);
  signal YCrCb_rg7            : std_logic_vector (9 downto 0);
  signal YCrCb_rg8            : std_logic_vector (9 downto 0);

  signal Fo                   : std_logic;
  signal Ho                   : std_logic;
  signal Vo                   : std_logic;
  
  signal MstReq2              : std_logic;
  signal MstReq3              : std_logic;
  signal NewData              : std_logic;
  --signal rst                  : std_logic;
  
begin
--  Reset and Clock input
--  27Mhz for SDTV
--  data from the input video stream
--  data delayed by pipe length
--  high = NTSC format detected
--  high = field one (even)
--  high = vertical blank
--  low = active video

-- -----------------------
--  Data type declarations
-- -----------------------

-- 
-- 
-- //-----------------------------------------------------------------------
--
--  TRS - Timing Reference Symbol = FF 00 00, in succession.

  process(YCrCb_rg5, YCrCb_rg6, YCrCb_rg7)
  begin
    if    YCrCb_rg5(9 downto 2) = "00000000" 
      and YCrCb_rg6(9 downto 2) = "00000000" 
      and YCrCb_rg7(9 downto 2) = "11111111" then
      TRS <= '1';
    else
      TRS <= '0';
    end if;
  end process;
  
  A <= Ho;
  B <= Vo;
  C <= Fo;

  IP2Bus_MstReq <= (NewData and not MstReq3) or MstReq2;
  NewData <= not vidcount(30);

  IP2Bus_Data <= pixeldata;
  IP2Bus_Addr <= (vidcount(0 to 29) & "00"); -- + C_FBADDR;
  IP2Bus_MstBE <= "1111";

  Ho <= H_rg(0) or H_rg(4) ;
  H_rising <= H_rg(0) and not H_rg(1) ;
  V_rising <= ( TRS and YCrCb_rg4(7) ) and not Vo ;
  V_falling <= ( TRS and not YCrCb_rg4(7) ) and Vo ;

  process (vid_clk)
  begin
    if  (Bus2IP_Reset = '1' ) then
      YCrCb_rg1 <= (others => '0') ;
      YCrCb_rg2 <= (others => '0') ;
      YCrCb_rg3 <= (others => '0') ;
      YCrCb_rg4 <= (others => '0') ;
      YCrCb_rg5 <= (others => '0') ;
      YCrCb_rg6 <= (others => '0') ;
      YCrCb_rg7 <= (others => '0') ;
      YCrCb_rg8 <= (others => '0') ;
      
      Fo <= '0' ;
      Vo <= '0' ;
      H_rg <= "00000";
      vidcount <= (others => '0');
    elsif vid_clk'event and vid_clk = '1' then
      vidcount <= vidcount + 1;
      
      YCrCb_rg1 <= YCrCb_in ;
      YCrCb_rg2 <= YCrCb_rg1 ;
      YCrCb_rg3 <= YCrCb_rg2 ;
      YCrCb_rg4 <= YCrCb_rg3 ;
      YCrCb_rg5 <= YCrCb_rg4 ;
      YCrCb_rg6 <= YCrCb_rg5 ;
      YCrCb_rg7 <= YCrCb_rg6 ;
      YCrCb_rg8 <= YCrCb_rg7 ;
      
      if  ( TRS = '1' ) then
        Fo <= YCrCb_rg4(8) ;
        Vo <= YCrCb_rg4(7) ;
        H_rg(4 downto 0) <= H_rg(4 downto 1) & YCrCb_rg4(6) ;
      else
        Fo <= Fo ;
        Vo <= Vo ;
        H_rg(4 downto 0) <= H_rg(3 downto 0) & H_rg(0) ;
      end if;
      
      if Ho = '1' then
        vidcount(30 to 31) <= "00";
      end if;
    end if;
    
    if V_falling = '1' and Fo = '0' then
      vidcount <= (others => '0');
    end if;
  end process;
  
  process (Bus2IP_Clk)
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      MstReq3 <= NewData and write_ena;
      
      if Bus2IP_MstAck = '1' then
        MstReq2 <= '0';
      else
        MstReq2 <= NewData and write_ena;
      end if;

    end if;
  end process;

  process (NewData)
  begin
    if Bus2IP_Reset = '1' then
      pixeldata <= (others => '0');
    elsif NewData'event and NewData = '1' then
      pixeldata <= YCrCb_rg5(9 downto 2) & 
                   YCrCb_rg6(9 downto 2) & 
                   YCrCb_rg7(9 downto 2) &
                   YCrCb_rg8(9 downto 2);
    end if;
  end process;

end architecture lf_decode_rtl;
