library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity vidcap is  --USER-- change entity name
  generic
  (
    C_OPB_AWIDTH         : INTEGER := 32;
    C_OPB_DWIDTH         : INTEGER := 32;
    C_DEV_BURST_ENABLE   : INTEGER := 0;
    C_DEV_MAX_BURST_SIZE : INTEGER := 64;
    C_FAMILY             : string  := "virtex2";
    C_FBADDR             : std_logic_vector(0 to 31) := X"00005000"
  );

  port
  (
    --Required OPB bus ports, do not add to or delete
    Mn_ABus           : out std_logic_vector(0 to C_OPB_AWIDTH - 1 );
    Mn_DBus           : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
    Mn_request        : out std_logic;
    Mn_busLock        : out std_logic;
    Mn_select         : out std_logic;
    Mn_RNW            : out std_logic;
    Mn_BE             : out std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 );
    Mn_seqAddr        : out std_logic;
    OPB_Clk           : in  std_logic := '0';
    OPB_Rst           : in  std_logic := '0';
    OPB_DBus          : in  std_logic_vector(0 to C_OPB_DWIDTH - 1 ) := (others => '0');
    OPB_MGrant        : in  std_logic := '0';
    OPB_xferAck       : in  std_logic := '0';
    OPB_errAck        : in  std_logic := '0';
    OPB_retry         : in  std_logic := '0';
    OPB_timeout       : in  std_logic := '0';
    
    --User ports
    led1              : out std_logic;
    led2              : out std_logic;
    YCrCb_in          : in std_logic_vector(9 downto 0);
    vid_clk           : in std_logic;
    write_ena         : in std_logic
  );
end entity vidcap;

architecture imp of vidcap is
  signal Mn_select_s : std_logic := '0';
  signal Mn_request_s : std_logic := '0';

  signal pixeldata            : std_logic_vector (31 downto 0) := X"00000000";
  signal vidcount	      : std_logic_vector (0 to 31);

  signal dataOut	      : std_logic_vector (0 to 31);
  signal addrOut	      : std_logic_vector (0 to 31);
  
  signal fifoA  	      : std_logic_vector (0 to 64);
  signal fifoB  	      : std_logic_vector (0 to 64);
  signal fifoC  	      : std_logic_vector (0 to 64);
  signal fifoD  	      : std_logic_vector (0 to 64);
  
  type fifoPtr is (Empty, A, B, C, D);
  signal nextWrite            : fifoPtr;

  signal DataRdy1             : std_logic;
  signal DataRdy2             : std_logic;
  signal NewData              : std_logic;

  signal H_rg                 : std_logic_vector (4 downto 0);
  signal H_rising             : std_logic;
  signal TRS                  : std_logic;
  signal V_falling            : std_logic;
  signal V_rising             : std_logic;
  signal YCrCb_rg1            : std_logic_vector (9 downto 0);
  signal YCrCb_rg2            : std_logic_vector (9 downto 0);
  signal YCrCb_rg3            : std_logic_vector (9 downto 0);
  signal YCrCb_rg4            : std_logic_vector (9 downto 0);
  signal YCrCb_rg5            : std_logic_vector (9 downto 0);

  signal Fo                   : std_logic;
  signal Ho                   : std_logic;
  signal Vo                   : std_logic;
begin
  led1 <= '0';
  led2 <= '0';

  Mn_busLock <= '0';
  Mn_RNW <= '0';
  Mn_BE <= (others => '1');
  Mn_seqAddr <= '0';
  
  Mn_request <= Mn_request_s;
  Mn_select <= Mn_select_s;
  

  process(OPB_Clk)
    variable addr : std_logic_vector(0 to C_OPB_AWIDTH - 1);
  begin
    if OPB_Clk'event and OPB_Clk = '1' then
    
      if OPB_xferAck = '1' then
        nextWrite <= C when fifoD(64) = '1' else
                     B when fifoC(64) = '1' else
                     A when fifoB(64) = '1' else Empty;
        

      -- reset signals
      if OPB_Rst = '1' then
        Mn_select_s <= '0';
        MstReq2 <= '0';
      elsif OPB_MGrant = '1' then
        MstReq2 <= '0';
        Mn_select_s <= '1';
      elsif MstReq3 = '0' then
        MstReq2 <= NewData;
      end if;
      
      if  or OPB_timeout = '1' or OPB_retry = '1' then
        Mn_select_s <= '0';
      end if;
      
      MstReq3 <= NewData;


      -- one OPB clock cycle delayed response
      if DataRdy1 = '1' and DataRdy2 = '0' then
        fifoD <= fifoC;
        fifoC <= fifoB;
        fifoB <= fifoA;
        fifoA(0 to 31) <= (vidcount(0 to 29) & "00") + C_FBADDR;
        fifoA(32 to 63) <= pixeldata;
        fifoA(64) <= '1';
      end if;
      
      if OPB_xferAck = '1' then
        fifoA 
      
      DataRdy2 <= DataRdy1;
      DataRdy1 <= NewData;
    end if;
  end process;
  
  
  TRS <= '1' when YCrCb_rg2(9 downto 2) = "00000000" 
              and YCrCb_rg3(9 downto 2) = "00000000" 
              and YCrCb_rg4(9 downto 2) = "11111111" else '0';
  
  Mn_request_s <= ((NewData and not MstReq3) or MstReq2) and not Mn_select_s;
  NewData <= write_ena and not (Ho or vidcount(30) or OPB_Rst);

  Mn_DBus <= pixeldata;
  Mn_ABus <= ;

  Ho <= H_rg(0) or H_rg(4) ;
  H_rising <= H_rg(0) and not H_rg(1) ;
  V_rising <= ( TRS and YCrCb_rg1(7) ) and not Vo ;
  V_falling <= ( TRS and not YCrCb_rg1(7) ) and Vo ;

  process (vid_clk)
  begin
    if  (OPB_Rst = '1' ) then
      YCrCb_rg1 <= (others => '0') ;
      YCrCb_rg2 <= (others => '0') ;
      YCrCb_rg3 <= (others => '0') ;
      YCrCb_rg4 <= (others => '0') ;
      YCrCb_rg5 <= (others => '0') ;
      
      Fo <= '0' ;
      Vo <= '0' ;
      H_rg <= "00000";
      vidcount <= (others => '0');
    elsif vid_clk'event and vid_clk = '1' then
      YCrCb_rg1 <= YCrCb_in ;
      YCrCb_rg2 <= YCrCb_rg1 ;
      YCrCb_rg3 <= YCrCb_rg2 ;
      YCrCb_rg4 <= YCrCb_rg3 ;
      YCrCb_rg5 <= YCrCb_rg4 ;
      
      if ( TRS = '1' ) then
        Fo <= YCrCb_rg1(8) ;
        Vo <= YCrCb_rg1(7) ;
        H_rg(4 downto 0) <= H_rg(4 downto 1) & YCrCb_rg1(6) ;
      else
        Fo <= Fo ;
        Vo <= Vo ;
        H_rg(4 downto 0) <= H_rg(3 downto 0) & H_rg(0) ;
      end if;
    
      if V_falling = '1' and Fo = '0' then
        vidcount <= (others => '0');
      elsif H_rg(4) = '1' and H_rg(3) = '0' then
        vidcount(30 to 31) <= "00";
      elsif Ho = '1' then
        vidcount(30) <= '1';
      else
        vidcount <= vidcount + 1;
      end if;

    end if;

  end process;
  
  process (NewData)
  begin
    if OPB_Rst = '1' then
      pixeldata <= (others => '0');
    elsif NewData'event and NewData = '1' then
      pixeldata <= YCrCb_rg2(9 downto 2) & 
                   YCrCb_rg3(9 downto 2) & 
                   YCrCb_rg4(9 downto 2) &
                   YCrCb_rg5(9 downto 2);
    end if;
  end process;
  
end architecture imp;
