-- $Id: vidcap.vhd,v 1.1 2003/06/27 01:51:47 anitas Exp $

-- vidcap.vhd
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--

library ieee;
use ieee.std_logic_1164.all;

library opb_ipif_msp0_v1_00_a;
use opb_ipif_msp0_v1_00_a.all;

--  Add user library if this source file uses
--  entities from the user library
library vidcap_v1_00_b; 
use vidcap_v1_00_b.all; 

-------------------------------------------------------------------------------
-- entity
-------------------------------------------------------------------------------

entity vidcap is 
  generic
  (
    C_OPB_AWIDTH         : INTEGER := 32;
    C_OPB_DWIDTH         : INTEGER := 32;
    C_DEV_BURST_ENABLE   : INTEGER := 0;
    C_DEV_MAX_BURST_SIZE : INTEGER := 64;
    C_FAMILY             : string  := "virtex2"
  );

  port
  (
    --Required OPB bus ports, do not add to or delete
    Mn_ABus           : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
    Mn_DBus           : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
    Mn_request        : out std_logic;
    Mn_busLock        : out std_logic;
    Mn_select         : out std_logic;
    Mn_RNW            : out std_logic;
    Mn_BE             : out std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 );
    Mn_seqAddr        : out std_logic;
    OPB_Clk           : in  std_logic := '0';
    OPB_Rst           : in  std_logic := '0';
    OPB_DBus          : in  std_logic_vector(0 to C_OPB_DWIDTH - 1 ) := (others => '0');
    OPB_MGrant        : in  std_logic := '0';
    OPB_xferAck       : in  std_logic := '0';
    OPB_errAck        : in  std_logic := '0';
    OPB_retry         : in  std_logic := '0';
    OPB_timeout       : in  std_logic := '0';

    -- Other control signals
    
    Freeze            : in  std_logic := '0';
    
    --USER-- add user I/Os to port list
    YCrCb_in          : in std_logic_vector(9 downto 0);
    vid_clk           : in std_logic
    rst               : in std_logic
    write_ena         : in std_logic
  );
end entity vidcap;

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of vidcap is

  --USER-- component declaration for user core logic. Change the
  --component declaration below to the declaration of the user's
  --logic.
  
  component lf_decode is
    generic
    (
      C_MEM_FILL_ADDR_START : std_logic_vector(0 to 31) := X"FFFFC000";
      C_MEM_FILL_ADDR_END   : std_logic_vector(0 to 31) := X"FFFFFFFF";
      C_REPEAT              : INTEGER := 1;
      C_OPB_AWIDTH          : INTEGER := 32;
      C_OPB_DWIDTH          : INTEGER := 32;
      C_DEV_BURST_ENABLE    : INTEGER := 0;
      C_DEV_MAX_BURST_SIZE  : INTEGER := 64;
      C_FAMILY              : string  := "virtex2"
    );
    port
    (
      Bus2IP_Clk        : in  std_logic;
      Bus2IP_Data       : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
      Bus2IP_Freeze     : in  std_logic;
      Bus2IP_IPMstTrans : in  std_logic;
      Bus2IP_MstAck     : in  std_logic;
      Bus2IP_MstRetry   : in  std_logic;
      Bus2IP_MstError   : in  std_logic;
      Bus2IP_MstTimeOut : in  std_logic;
      Bus2IP_MstLastAck : in  std_logic;
      Bus2IP_Reset      : in  std_logic;
      IP2Bus_Addr       : out std_logic_vector(0 to C_OPB_AWIDTH - 1 ) := (others => '0');
      IP2Bus_Data       : out std_logic_vector(0 to C_OPB_DWIDTH -1) := (others => '0');
      IP2Bus_MstBE      : out std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 ) := (others => '0');
      IP2Bus_MstReq     : out std_logic := '0';
      IP2Bus_MstRNW     : out std_logic := '0';
      IP2Bus_MstBurst   : out std_logic := '0';
      IP2Bus_MstBusLock : out std_logic := '0'
      YCrCb_in          : in std_logic_vector(9 downto 0);
      clk               : in std_logic
      rst               : in std_logic
      write_ena         : in std_logic
    );
  end component lf_decode;

  --OPB_IPIF_MSP0 declaration -- do not change
  component opb_ipif_msp0 is
    generic
    (
      C_OPB_AWIDTH         : INTEGER := 32;
      C_OPB_DWIDTH         : INTEGER := 32;
      C_DEV_BURST_ENABLE   : INTEGER := 0;
      C_DEV_MAX_BURST_SIZE : INTEGER := 64;
      C_FAMILY             : string  := "virtex2"
    );
    port
    (
      Mn_ABus           : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
      Mn_DBus           : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
      Mn_request        : out std_logic;
      Mn_busLock        : out std_logic;
      Mn_select         : out std_logic;
      Mn_RNW            : out std_logic;
      Mn_BE             : out std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 );
      Mn_seqAddr        : out std_logic;
      OPB_Clk           : in  std_logic := '0';
      OPB_Rst           : in  std_logic := '0';
      OPB_DBus          : in  std_logic_vector(0 to C_OPB_DWIDTH - 1 ) := (others => '0');
      OPB_MGrant        : in  std_logic := '0';
      OPB_xferAck       : in  std_logic := '0';
      OPB_errAck        : in  std_logic := '0';
      OPB_retry         : in  std_logic := '0';
      OPB_timeout       : in  std_logic := '0';
      Bus2IP_Clk        : out std_logic;
      Bus2IP_Data       : out std_logic_vector(0 to C_OPB_DWIDTH-1);
      Bus2IP_Freeze     : out std_logic;
      Bus2IP_IPMstTrans : out std_logic;
      Bus2IP_MstAck     : out std_logic;
      Bus2IP_MstRetry   : out std_logic;
      Bus2IP_MstError   : out std_logic;
      Bus2IP_MstTimeOut : out std_logic;
      Bus2IP_MstLastAck : out std_logic;
      Bus2IP_Reset      : out std_logic;
      IP2Bus_Addr       : in  std_logic_vector(0 to C_OPB_AWIDTH - 1 ) := (others => '0');
      IP2Bus_Data       : in  std_logic_vector(0 to C_OPB_DWIDTH -1) := (others => '0');
      IP2Bus_MstBE      : in  std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 ) := (others => '0');
      IP2Bus_MstReq     : in  std_logic := '0';
      IP2Bus_MstRNW     : in  std_logic := '0';
      IP2Bus_MstBurst   : in  std_logic := '0';
      IP2Bus_MstBusLock : in  std_logic := '0';
      Freeze            : in  std_logic := '0'
    );
  end component opb_ipif_msp0;

  --IP Interconnect (IPIC) signal list --do not delete
  signal Bus2IP_Clk        : std_logic;
  signal Bus2IP_Data       : std_logic_vector(0 to C_OPB_DWIDTH-1);
  signal Bus2IP_Freeze     : std_logic;
  signal Bus2IP_IPMstTrans : std_logic;
  signal Bus2IP_MstAck     : std_logic;
  signal Bus2IP_MstRetry   : std_logic;
  signal Bus2IP_MstError   : std_logic;
  signal Bus2IP_MstTimeOut : std_logic;
  signal Bus2IP_MstLastAck : std_logic;
  signal Bus2IP_Reset      : std_logic;
  signal IP2Bus_Addr       : std_logic_vector(0 to C_OPB_AWIDTH - 1 ) := (others => '0');
  signal IP2Bus_Data       : std_logic_vector(0 to C_OPB_DWIDTH -1) := (others => '0');
  signal IP2Bus_MstBE      : std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 ) := (others => '0');
  signal IP2Bus_MstReq     : std_logic := '0';
  signal IP2Bus_MstRNW     : std_logic := '0';
  signal IP2Bus_MstBurst   : std_logic := '0';
  signal IP2Bus_MstBusLock : std_logic := '0';
         
------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------

  --OPB IPIF lite instantiation --do not delete
  OPB_IPIF_MSP0_I : opb_ipif_msp0
    generic map
    (
      C_OPB_AWIDTH         => C_OPB_AWIDTH,
      C_OPB_DWIDTH         => C_OPB_DWIDTH,
      C_DEV_BURST_ENABLE   => C_DEV_BURST_ENABLE,
      C_DEV_MAX_BURST_SIZE => C_DEV_MAX_BURST_SIZE,
      C_FAMILY             => C_FAMILY
    )
    port map
    (
      Mn_ABus           => Mn_ABus,
      Mn_DBus           => Mn_DBus,
      Mn_request        => Mn_request,
      Mn_busLock        => Mn_busLock,
      Mn_select         => Mn_select,
      Mn_RNW            => Mn_RNW,
      Mn_BE             => Mn_BE,
      Mn_seqAddr        => Mn_seqAddr,
      OPB_Clk           => OPB_Clk,
      OPB_Rst           => OPB_Rst,
      OPB_DBus          => OPB_DBus,
      OPB_MGrant        => OPB_MGrant,
      OPB_xferAck       => OPB_xferAck,
      OPB_errAck        => OPB_errAck,
      OPB_retry         => OPB_retry,
      OPB_timeout       => OPB_timeout,
      Bus2IP_Clk        => Bus2IP_Clk,
      Bus2IP_Data       => Bus2IP_Data,
      Bus2IP_Freeze     => Bus2IP_Freeze,
      Bus2IP_IPMstTrans => Bus2IP_IPMstTrans,
      Bus2IP_MstAck     => Bus2IP_MstAck,
      Bus2IP_MstRetry   => Bus2IP_MstRetry,
      Bus2IP_MstError   => Bus2IP_MstError,
      Bus2IP_MstTimeOut => Bus2IP_MstTimeOut,
      Bus2IP_MstLastAck => Bus2IP_MstLastAck,
      Bus2IP_Reset      => Bus2IP_Reset,
      IP2Bus_Addr       => IP2Bus_Addr,
      IP2Bus_Data       => IP2Bus_Data,
      IP2Bus_MstBE      => IP2Bus_MstBE,
      IP2Bus_MstReq     => IP2Bus_MstReq,
      IP2Bus_MstRNW     => IP2Bus_MstRNW,
      IP2Bus_MstBurst   => IP2Bus_MstBurst,
      IP2Bus_MstBusLock => IP2Bus_MstBusLock,
      Freeze            => Freeze
    );

  USER_LOGIC_MASTER0_I : lf_decode
    generic map
    (
      C_MEM_FILL_ADDR_START => X"10000000",
      C_MEM_FILL_ADDR_END   => X"1000FFFF",
      C_REPEAT              => 1,
      C_OPB_AWIDTH          => C_OPB_AWIDTH,
      C_OPB_DWIDTH          => C_OPB_DWIDTH,
      C_DEV_BURST_ENABLE    => C_DEV_BURST_ENABLE,
      C_DEV_MAX_BURST_SIZE  => C_DEV_MAX_BURST_SIZE,
      C_FAMILY              => C_FAMILY
    )
    port map
    (
      Bus2IP_Clk        => Bus2IP_Clk,
      Bus2IP_Data       => Bus2IP_Data,
      Bus2IP_Freeze     => Bus2IP_Freeze,
      Bus2IP_IPMstTrans => Bus2IP_IPMstTrans,
      Bus2IP_MstAck     => Bus2IP_MstAck,
      Bus2IP_MstRetry   => Bus2IP_MstRetry,
      Bus2IP_MstError   => Bus2IP_MstError,
      Bus2IP_MstTimeOut => Bus2IP_MstTimeOut,
      Bus2IP_MstLastAck => Bus2IP_MstLastAck,
      Bus2IP_Reset      => Bus2IP_Reset,
      IP2Bus_Addr       => IP2Bus_Addr,
      IP2Bus_Data       => IP2Bus_Data,
      IP2Bus_MstBE      => IP2Bus_MstBE,
      IP2Bus_MstReq     => IP2Bus_MstReq,
      IP2Bus_MstRNW     => IP2Bus_MstRNW,
      IP2Bus_MstBurst   => IP2Bus_MstBurst,
      IP2Bus_MstBusLock => IP2Bus_MstBusLock,
      YCrCb_in          => YCrCb_in,
      clk               => vid_clk,
      rst               => rst,
      write_ena         => write_ena
    );

end architecture imp;

